pub struct Formula { /* private fields */ }Expand description
A microgrid metric formula generated by traversing the component graph.
Formulas represent quantities derived from multiple components — both
aggregated metrics like power or current (which can be added and
subtracted), and coalesced metrics like AC voltage or frequency. They can be
combined with +, -, coalesce, min and
max, and converted to a string before being passed to an
evaluator.
Implementations